// Driver for LCD Nokia 3310 


#include <mega128.h>
#include <delay.h>
#include <stdio.h>
#include <LCD_3310.h>
#include <spi.h>

unsigned char buff[6][84];
unsigned char posx, posy;
//unsigned char ctr= 8;

//     -  
 #define Nokia_3310  // ,  -  
 

 #define SPI_P  //    SPI    

 
//      

#define LCD_DC PORTC.5
#define LCD_RESET PORTC.4

#ifdef SPI_P 
    void sp(char data_to_lcd)
        {
        spi(data_to_lcd);
        };
#else

    #define LCD_CLOCK_LINE PORTB.1
    #define LCD_DATA_LINE PORTB.2

    void sp(char data_to_lcd)  //  SPI
        {
        char ctr= 8;
        //ctr= 8;
        while(ctr)  
            {
            LCD_DATA_LINE = data_to_lcd & 0b10000000;
            data_to_lcd <<= 1;
            ctr --;
            LCD_CLOCK_LINE = 1;
            LCD_CLOCK_LINE = 0;
            };
        }

#endif



#ifdef Nokia_3310  // 
    void LCD_init(void) //  
    {
    LCD_DC =0;
    LCD_RESET=0;
    delay_ms(1);
    LCD_RESET=1;
    delay_ms(1);
    LCD_DC=0;
    sp(0x21);     //  , . 
    sp(0x00);     //   , Vop=0x48;
    sp(0x06);    
    sp(0x13);      // bias=3
    sp(0x20);     //   
    sp(0x0c);     
    delay_us(500);
    LCD_clear();  
    LCD_refresh();  
    }
    // #endif

    //------------------------------------------------------
    void LCD_refresh (void)  //   ( )
    {
    unsigned char i,j;

    LCD_DC=0 ;
    sp(64);
    LCD_DC=1;
    for(i=0; i<6; i++)
      { 
      LCD_DC=0 ;
      sp(64+i);
      sp(128);
      LCD_DC=1;
      for (j=0; j<84; j++) sp(buff[i][j]);
      }
    }
    //------------------------------------------------------

    //------------------------------------------------------
    void LCD_pixel (unsigned char x, unsigned char y, unsigned char color)
    {   //  .      
    unsigned char i, bt;
    i=y>>3;
    bt=1<<(y & 0x7);
    if(color) buff[i][x]|=bt;
    else buff[i][x]&=(~bt); 
    }
    //------------------------------------------------------

    //------------------------------------------------------
    void LCD_clear (void)
    {
    unsigned char x, y;
    for (y=0; y<6; y++)    
      for (x=0; x<84; x++) buff[y][x]=0;   
    } 


#else // Nokia_3310_China  //  
    void LCD_init(void) //  
    {
    LCD_DC =0;
    LCD_RESET=0;
    delay_ms(1);
    LCD_RESET=1;
    delay_ms(1);
    LCD_DC=0;
    sp(0x21);     //  , . 
    sp(0xD8);     //   ,  
    // (0xC8)   , Vop=0x48;      spi(0xF4);
    //    !!!!!!!!!!
    // F4 -  
    // A4 -   
    // F0 - +
    // F2 - 

    // E0 -  !
    // EB -  !
    // E8 -   !

    sp(0x45);     // New for new disp
    sp(0x06);    
    sp(0x13);      // bias=3
    sp(0x20);     //   
    sp(0x0c);     
    delay_us(500);
    LCD_clear();  
    LCD_refresh();  
    }




    //------------------------------------------------------
    void LCD_refresh (void)  //   ( )
    {
    unsigned char i,j;

    LCD_DC=0 ;
    sp(64);
    LCD_DC=1;
    for(i=0; i<7; i++)
      { 
      LCD_DC=0 ;
      sp(64+i);
      sp(128);
      LCD_DC=1;
      for (j=0; j<84; j++) sp(buff[i][j]);
      }
    }



    //------------------------------------------------------
    void LCD_pixel (unsigned char x, unsigned char y, unsigned char color)
    {   //  .      
    unsigned char i, bt;
    y=y+8;
    i=y>>3;
    bt=1<<(y & 0x7);
    if(color) buff[i][x]|=bt;
    else buff[i][x]&=(~bt); 
    }


    //------------------------------------------------------
    void LCD_clear (void)
    {
    unsigned char x, y;
    for (y=0; y<7; y++)    
      for (x=0; x<84; x++) buff[y][x]=0;   
    } 


#endif

          

//------------------------------------------------------
void LCD_line ( char x1, char y1, char x2, char y2, char mode )
{
int dx, dy, stepx, stepy, fraction;

dy = y2 - y1;
dx = x2 - x1;
if ( dy < 0 ) { dy= -dy; stepy= -1; }
else stepy = 1;
if ( dx < 0 ) { dx= -dx; stepx= -1; }
else stepx = 1;
dx <<= 1;
dy <<= 1;
LCD_pixel( x1, y1, mode );
if ( dx > dy )
  {
  fraction = dy - (dx >> 1);
  while ( x1 != x2 )
    {
    if ( fraction >= 0 ) { y1 += stepy; fraction -= dx; }
    x1 += stepx;
    fraction += dy;
    LCD_pixel( x1, y1, mode );
    }
  }
else
  {
  fraction = dx - (dy >> 1);
  while ( y1 != y2 )
    {
    if ( fraction >= 0 ) { x1 += stepx; fraction -= dy; }
    y1 += stepy;
    fraction += dx;
    LCD_pixel( x1, y1, mode );
    }
  }
}
//------------------------------------------------------
void LCD_dashline ( char x1, char y1, char x2, char y2)
{
char mode;
int dx, dy, stepx, stepy, fraction;

dy = y2 - y1;
dx = x2 - x1;
mode=1;
    
if ( dy < 0 ) { dy= -dy; stepy = -1; }
else stepy = 1;
if ( dx < 0 ) { dx= -dx; stepx = -1; }
else stepx = 1;
dx <<= 1;
dy <<= 1;
LCD_pixel( x1, y1, mode );
if ( dx > dy )
  {
  fraction = dy - (dx >> 1);
  while ( x1 != x2 )
    {
    if(fraction >= 0 ) { y1 += stepy; fraction -= dx; }
    x1 += stepx;
    fraction += dy;
    if (mode) mode=0;
    else mode=1;
    LCD_pixel( x1, y1, mode );
    }
  }
else
  {
  fraction = dx - (dy >> 1);
  while ( y1 != y2 )
    {
    if ( fraction >= 0 ) { x1 += stepx; fraction -= dy; }
    y1 += stepy;
    fraction += dx;
    if (mode) mode=0;
    else mode=1;
    LCD_pixel( x1, y1, mode );
    }
  }
}
//------------------------------------------------------
void LCD_rectangle(char x, char y, char x1, char y1,unsigned char color)  // 
{
LCD_line (x,y,x,y1,color);
LCD_line (x,y1,x1,y1,color);
LCD_line (x1,y1,x1,y,color);
LCD_line (x,y,x1,y,color);
}
//------------------------------------------------------
void LCD_fillrect  (char x, char y, char x1, char y1,unsigned char color)  //  
{
unsigned char i;
for (i=y; i<=y1; i++) LCD_line(x,i,x1,i,color);
}

//  
__flash const unsigned char font_norm[][9] = {         

    {0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  
    {0x03,0x5F,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  !
    {0x04,0x03,0x00,0x03,0x00,0x00,0x00,0x00,0x00},    //  "
    {0x07,0x24,0x7E,0x24,0x24,0x7E,0x24,0x00,0x00},    //  #
    {0x06,0x04,0x2A,0x7F,0x2A,0x10,0x00,0x00,0x00},    //  $
    {0x08,0x02,0x05,0x62,0x18,0x26,0x50,0x20,0x00},    //  %
    {0x07,0x36,0x49,0x49,0x56,0x20,0x50,0x00,0x00},    //  &
    {0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  
    {0x03,0x3E,0x41,0x00,0x00,0x00,0x00,0x00,0x00},    //  (
    {0x03,0x41,0x3E,0x00,0x00,0x00,0x00,0x00,0x00},    //  )
    {0x06,0x14,0x08,0x3E,0x08,0x14,0x00,0x00,0x00},    //  *
    {0x06,0x08,0x08,0x3E,0x08,0x08,0x00,0x00,0x00},    //  +
    {0x04,0x80,0x60,0x00,0x00,0x00,0x00,0x00,0x00},    //  ,
    {0x05,0x08,0x08,0x08,0x08,0x00,0x00,0x00,0x00},    //  -
    {0x02,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  .
    {0x04,0x60,0x1C,0x03,0x00,0x00,0x00,0x00,0x00},    //  /
    {0x06,0x3E,0x41,0x41,0x41,0x3E,0x00,0x00,0x00},    //  0   48   
    {0x06,0x04,0x02,0x7F,0x00,0x00,0x00,0x00,0x00},    //  1
    {0x06,0x42,0x61,0x51,0x49,0x46,0x00,0x00,0x00},    //  2   50
    {0x06,0x22,0x41,0x49,0x49,0x36,0x00,0x00,0x00},    //  3
    {0x06,0x18,0x14,0x12,0x7F,0x10,0x00,0x00,0x00},    //  4
    {0x06,0x27,0x45,0x45,0x45,0x39,0x00,0x00,0x00},    //  5
    {0x06,0x3E,0x45,0x45,0x45,0x38,0x00,0x00,0x00},    //  6
    {0x06,0x01,0x61,0x19,0x07,0x01,0x00,0x00,0x00},    //  7
    {0x06,0x36,0x49,0x49,0x49,0x36,0x00,0x00,0x00},    //  8
    {0x06,0x0E,0x51,0x51,0x51,0x3E,0x00,0x00,0x00},    //  9
    {0x03,0x22,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  :
    {0x04,0x80,0x62,0x00,0x00,0x00,0x00,0x00,0x00},    //  ;
    {0x04,0x08,0x14,0x22,0x00,0x00,0x00,0x00,0x00},    //  <  60
    {0x05,0x14,0x14,0x14,0x14,0x00,0x00,0x00,0x00},    //  =
    {0x04,0x22,0x14,0x08,0x00,0x00,0x00,0x00,0x00},    //  >
    {0x06,0x02,0x01,0x51,0x09,0x06,0x00,0x00,0x00},    //  ?
    {0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  
    {0x06,0x7C,0x12,0x11,0x12,0x7C,0x00,0x00,0x00},    //  A
    {0x06,0x7F,0x49,0x49,0x49,0x3E,0x00,0x00,0x00},    //  B   
    {0x06,0x3E,0x41,0x41,0x41,0x22,0x00,0x00,0x00},    //  C
    {0x06,0x7F,0x41,0x41,0x22,0x1C,0x00,0x00,0x00},    //  D 
    {0x06,0x7F,0x49,0x49,0x49,0x49,0x00,0x00,0x00},    //  E
    {0x05,0x7F,0x09,0x09,0x01,0x00,0x00,0x00,0x00},    //  F  70
    {0x06,0x3C,0x42,0x41,0x51,0x72,0x00,0x00,0x00},    //  G
    {0x06,0x7F,0x08,0x08,0x08,0x7F,0x00,0x00,0x00},    //  H
    {0x03,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  I
    {0x06,0x30,0x40,0x40,0x40,0x3F,0x00,0x00,0x00},    //  J
    {0x06,0x7F,0x08,0x14,0x22,0x41,0x00,0x00,0x00},    //  K
    {0x06,0x7F,0x40,0x40,0x40,0x40,0x00,0x00,0x00},    //  L
	{0x07,0x7F,0x02,0x04,0x04,0x02,0x7F,0x00,0x00},    //  M
	{0x06,0x7F,0x04,0x08,0x10,0x7F,0x00,0x00,0x00},    //  N
	{0x06,0x3E,0x41,0x41,0x41,0x3E,0x00,0x00,0x00},    //  O
	{0x06,0x7F,0x09,0x09,0x09,0x06,0x00,0x00,0x00},    //  P  80
	{0x06,0x3E,0x41,0x61,0xC1,0x3E,0x00,0x00,0x00},    //  Q
	{0x06,0x7F,0x09,0x09,0x19,0x66,0x00,0x00,0x00},    //  R
	{0x06,0x26,0x49,0x49,0x49,0x32,0x00,0x00,0x00},    //  S
	{0x06,0x01,0x01,0x7F,0x01,0x01,0x00,0x00,0x00},    //  T
	{0x06,0x3F,0x40,0x40,0x40,0x3F,0x00,0x00,0x00},    //  U
	{0x07,0x07,0x38,0x40,0x40,0x38,0x07,0x00,0x00},    //  V
	{0x08,0x3F,0x40,0x38,0x06,0x38,0x40,0x3F,0x00},    //  W
	{0x06,0x63,0x14,0x08,0x14,0x63,0x00,0x00,0x00},    //  X
	{0x06,0x03,0x04,0x78,0x04,0x03,0x00,0x00,0x00},    //  Y
	{0x06,0x61,0x51,0x49,0x45,0x43,0x00,0x00,0x00},    //  Z  90
	{0x03,0x7F,0x41,0x00,0x00,0x00,0x00,0x00,0x00},    //  [
	{0x04,0x03,0x1C,0x60,0x00,0x00,0x00,0x00,0x00},    //  \
	{0x03,0x41,0x7F,0x00,0x00,0x00,0x00,0x00,0x00},    //  ]
	{0x06,0x04,0x02,0x01,0x02,0x04,0x00,0x00,0x00},    //  ^
	{0x05,0x80,0x80,0x80,0x80,0x00,0x00,0x00,0x00},    //  _
	{0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  
	{0x05,0x20,0x54,0x54,0x78,0x00,0x00,0x00,0x00},    //  a
	{0x05,0x7F,0x44,0x44,0x38,0x00,0x00,0x00,0x00},    //  b
	{0x05,0x38,0x44,0x44,0x44,0x00,0x00,0x00,0x00},    //  c
	{0x05,0x38,0x44,0x44,0x7F,0x00,0x00,0x00,0x00},    //  d  100
	{0x05,0x38,0x54,0x54,0x58,0x00,0x00,0x00,0x00},    //  e
	{0x04,0x04,0x7E,0x05,0x00,0x00,0x00,0x00,0x00},    //  f
	{0x05,0x18,0xA4,0xA4,0x7C,0x00,0x00,0x00,0x00},    //  g
	{0x05,0x7F,0x04,0x04,0x78,0x00,0x00,0x00,0x00},    //  h
	{0x02,0x7D,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    //  i
	{0x04,0x40,0x80,0x7D,0x00,0x00,0x00,0x00,0x00},    //  j
	{0x05,0x7F,0x10,0x28,0x44,0x00,0x00,0x00,0x00},    //  k
	{0x03,0x7F,0x40,0x00,0x00,0x00,0x00,0x00,0x00},    //  l
	{0x06,0x7C,0x04,0x7C,0x04,0x78,0x00,0x00,0x00},    //  m
	{0x05,0x7C,0x04,0x04,0x78,0x00,0x00,0x00,0x00},    //  n  110
	{0x05,0x38,0x44,0x44,0x38,0x00,0x00,0x00,0x00},    //  o
	{0x05,0xFC,0x24,0x24,0x18,0x00,0x00,0x00,0x00},    //  p
	{0x05,0x18,0x24,0x24,0xFC,0x80,0x00,0x00,0x00},    //  q
	{0x04,0x78,0x04,0x04,0x00,0x00,0x00,0x00,0x00},    //  r
	{0x05,0x48,0x54,0x54,0x24,0x00,0x00,0x00,0x00},    //  s
	{0x04,0x04,0x7F,0x44,0x00,0x00,0x00,0x00,0x00},    //  t
	{0x05,0x3C,0x40,0x40,0x7C,0x00,0x00,0x00,0x00},    //  u
	{0x06,0x04,0x38,0x40,0x38,0x04,0x00,0x00,0x00},    //  v
	{0x08,0x3C,0x40,0x30,0x08,0x30,0x40,0x3C,0x00},    //  w
	{0x06,0x44,0x28,0x10,0x28,0x44,0x00,0x00,0x00},    //  x
	{0x06,0x04,0x98,0x60,0x18,0x04,0x00,0x00,0x00},    //  y
	{0x06,0x44,0x64,0x54,0x4C,0x44,0x00,0x00,0x00},    //  z
    {0x08, 0xFF,0x81,0x81,0x81,0x81,0x81,0x81,0xFF}, //       123
    {0x08, 0xFF,0x81,0xBD,0xBD,0xBD,0xBD,0x81,0xFF}, //       124
    {0x08, 0x01,0x31,0x79,0x79,0x7D,0x7F,0x7D,0x31},  // 125,126 ; 
    {0x08, 0x11,0x11,0x11,0x10,0x10,0x10,0x38,0x10},  
    {0x08, 0x3E,0x08,0x1C,0x1C,0x7C,0x3E,0x1E,0x1E},  // 127,128 ;
    {0x08, 0x1E,0x1C,0x18,0x18,0x18,0x1C,0x0E,0x0E},
    {0x06, 0x07,0x05,0x05,0x05,0x07,0x02,0x00,0x00}, // batt 129
    {0x05, 0x01,0x02,0x0f,0x02,0x01,0x00,0x00,0x00}, // antenn 130
    {0x05, 0x06,0x09,0x09,0x06,0x00,0x00,0x00,0x00}, // t-gradC 131
    {0x08, 0x0a,0xfe,0xab,0xff,0xfe,0xea,0xe0,0x40}, // RPM 132
    {0x08, 0x3c,0x42,0x85,0x99,0x99,0x85,0x42,0x3c}, // Clock 133
    {0x08, 0x20,0x30,0x38,0x3c,0x38,0x30,0x20,0x00}, // arrow up 134
    {0x08, 0x04,0x0c,0x1c,0x3c,0x1c,0x0c,0x04,0x00}, // arrow down 135
    {0x08, 0x04,0x04,0x7c,0x40,0x40,0x7c,0x04,0x04}, // PPM 136
    {0x08, 0x40,0x40,0x7c,0x04,0x04,0x7c,0x40,0x40}, // iPPM 137
	{0x08,0x3F,0x40,0x38,0x06,0x38,0x40,0x3F,0x00},    
	{0x06,0x63,0x14,0x08,0x14,0x63,0x00,0x00,0x00},    
	{0x06,0x03,0x04,0x78,0x04,0x03,0x00,0x00,0x00},    
	{0x06,0x61,0x51,0x49,0x45,0x43,0x00,0x00,0x00},    
	{0x03,0x7F,0x41,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x04,0x03,0x1C,0x60,0x00,0x00,0x00,0x00,0x00},   
	{0x03,0x41,0x7F,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x06,0x04,0x02,0x01,0x02,0x04,0x00,0x00,0x00},    
	{0x05,0x80,0x80,0x80,0x80,0x00,0x00,0x00,0x00},     
	{0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x05,0x20,0x54,0x54,0x78,0x00,0x00,0x00,0x00},    
	{0x05,0x7F,0x44,0x44,0x38,0x00,0x00,0x00,0x00},    
	{0x05,0x38,0x44,0x44,0x44,0x00,0x00,0x00,0x00},    
	{0x05,0x38,0x44,0x44,0x7F,0x00,0x00,0x00,0x00},    
	{0x05,0x38,0x54,0x54,0x58,0x00,0x00,0x00,0x00},    
	{0x04,0x04,0x7E,0x05,0x00,0x00,0x00,0x00,0x00},    
	{0x05,0x18,0xA4,0xA4,0x7C,0x00,0x00,0x00,0x00},    
	{0x05,0x7F,0x04,0x04,0x78,0x00,0x00,0x00,0x00},    
	{0x02,0x7D,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x04,0x40,0x80,0x7D,0x00,0x00,0x00,0x00,0x00},    
	{0x05,0x7F,0x10,0x28,0x44,0x00,0x00,0x00,0x00},     
	{0x03,0x7F,0x40,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x06,0x7C,0x04,0x7C,0x04,0x78,0x00,0x00,0x00},    
	{0x05,0x7C,0x04,0x04,0x78,0x00,0x00,0x00,0x00},    
	{0x05,0x38,0x44,0x44,0x38,0x00,0x00,0x00,0x00},    
	{0x05,0xFC,0x24,0x24,0x18,0x00,0x00,0x00,0x00},    
	{0x05,0x18,0x24,0x24,0xFC,0x80,0x00,0x00,0x00},    
	{0x04,0x78,0x04,0x04,0x00,0x00,0x00,0x00,0x00},    
	{0x05,0x48,0x54,0x54,0x24,0x00,0x00,0x00,0x00},   
	{0x04,0x04,0x7F,0x44,0x00,0x00,0x00,0x00,0x00},    
	{0x05,0x3C,0x40,0x40,0x7C,0x00,0x00,0x00,0x00},    
	{0x06,0x04,0x38,0x40,0x38,0x04,0x00,0x00,0x00},    
	{0x08,0x3C,0x40,0x30,0x08,0x30,0x40,0x3C,0x00},    
	{0x06,0x44,0x28,0x10,0x28,0x44,0x00,0x00,0x00},   
	{0x06,0x04,0x98,0x60,0x18,0x04,0x00,0x00,0x00},    
	{0x06,0x44,0x64,0x54,0x4C,0x44,0x00,0x00,0x00},
    {0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x03,0x5F,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x04,0x03,0x00,0x03,0x00,0x00,0x00,0x00,0x00},    
	{0x07,0x24,0x7E,0x24,0x24,0x7E,0x24,0x00,0x00},   
	{0x06,0x04,0x2A,0x7F,0x2A,0x10,0x00,0x00,0x00},    
	{0x08,0x02,0x05,0x62,0x18,0x26,0x50,0x20,0x00},  
    {0x08,0x02,0x05,0x62,0x18,0x26,0x50,0x20,0x00},
	{0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x03,0x3E,0x41,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x03,0x41,0x3E,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x06,0x14,0x08,0x3E,0x08,0x14,0x00,0x00,0x00},    
	{0x06,0x08,0x08,0x3E,0x08,0x08,0x00,0x00,0x00},    
	{0x04,0x80,0x60,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x05,0x08,0x08,0x08,0x08,0x00,0x00,0x00,0x00},    
	{0x03,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00},    
	{0x04,0x60,0x1C,0x03,0x00,0x00,0x00,0x00,0x00},    
    {0x06,0x3E,0x41,0x41,0x41,0x3E,0x00,0x00,0x00},      
	{0x06,0x04,0x02,0x7F,0x00,0x00,0x00,0x00,0x00},    
	{0x06,0x7C, 0x12, 0x11, 0x12, 0x7C,0x00,0x00,0x00}, //     192
	{0x06,0x7F, 0x49, 0x49, 0x49, 0x31,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x49, 0x49, 0x49, 0x36,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x01, 0x01, 0x01, 0x01,0x00,0x00,0x00}, //   
	{0x06,0x60, 0x3F, 0x21, 0x3F, 0x60,0x00,0x00,0x00}, //          
	{0x06,0x7F, 0x49, 0x49, 0x49, 0x41,0x00,0x00,0x00}, //   
	{0x06,0x77, 0x08, 0x7F, 0x08, 0x77,0x00,0x00,0x00}, //   
	{0x06,0x22, 0x41, 0x49, 0x49, 0x36,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x10, 0x08, 0x04, 0x7F,0x00,0x00,0x00}, //         200
	{0x06,0x7E, 0x10, 0x09, 0x04, 0x7E,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x08, 0x14, 0x22, 0x41,0x00,0x00,0x00}, //   
	{0x06,0x40, 0x3E, 0x01, 0x01, 0x7F,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x02, 0x0C, 0x02, 0x7F,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x08, 0x08, 0x08, 0x7F,0x00,0x00,0x00}, //   
	{0x06,0x3E, 0x41, 0x41, 0x41, 0x3E,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x01, 0x01, 0x01, 0x7F,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x09, 0x09, 0x09, 0x06,0x00,0x00,0x00}, //     
	{0x06,0x3E, 0x41, 0x41, 0x41, 0x22,0x00,0x00,0x00}, //   
	{0x06,0x01, 0x01, 0x7F, 0x01, 0x01,0x00,0x00,0x00}, //    
	{0x06,0x07, 0x48, 0x48, 0x48, 0x3F,0x00,0x00,0x00}, //   
	{0x06,0x0E, 0x11, 0x7F, 0x11, 0x0E,0x00,0x00,0x00}, //   
	{0x06,0x63, 0x14, 0x08, 0x14, 0x63,0x00,0x00,0x00}, //   
	{0x06,0x3F, 0x20, 0x20, 0x3F, 0x60,0x00,0x00,0x00}, //   
	{0x06,0x07, 0x08, 0x08, 0x08, 0x7F,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x40, 0x7E, 0x40, 0x7F,0x00,0x00,0x00}, //   
	{0x06,0x3F, 0x20, 0x3F, 0x20, 0x7F,0x00,0x00,0x00}, //  
    {0x06,0x01, 0x7F, 0x48, 0x48, 0x30,0x00,0x00,0x00}, // 
	{0x06,0x7F, 0x48, 0x30, 0x00, 0x7F,0x00,0x00,0x00}, //   
	{0x06,0x00, 0x7F, 0x48, 0x48, 0x30,0x00,0x00,0x00}, //  
	{0x06,0x22, 0x41, 0x49, 0x49, 0x3E,0x00,0x00,0x00}, //   
	{0x06,0x7F, 0x08, 0x3E, 0x41, 0x3E,0x00,0x00,0x00}, //  
	{0x06,0x46, 0x29, 0x19, 0x09, 0x7F,0x00,0x00,0x00}, //  
	{0x06,0x20, 0x54, 0x54, 0x54, 0x78,0x00,0x00,0x00}, //         
	{0x06,0x3C, 0x4A, 0x4A, 0x4A, 0x31,0x00,0x00,0x00}, //        225
	{0x05,0x7C, 0x54, 0x54, 0x28, 0x00,0x00,0x00,0x00}, //  
	{0x05,0x7C, 0x04, 0x04, 0x0C, 0x00,0x00,0x00,0x00}, // 
    {0x06,0x60, 0x3C, 0x24, 0x3C, 0x60,0x00,0x00,0x00}, //   
	{0x06,0x38, 0x54, 0x54, 0x54, 0x18,0x00,0x00,0x00}, //   
	{0x06,0x6C, 0x10, 0x7C, 0x10, 0x6C,0x00,0x00,0x00}, //   
	{0x06,0x00, 0x44, 0x54, 0x54, 0x28,0x00,0x00,0x00}, //   
	{0x06,0x7C, 0x20, 0x10, 0x08, 0x7C,0x00,0x00,0x00}, //   
	{0x06,0x7C, 0x21, 0x12, 0x09, 0x7C,0x00,0x00,0x00}, //   
	{0x05,0x7C, 0x10, 0x28, 0x44, 0x00,0x00,0x00,0x00}, //    
	{0x06,0x40, 0x38, 0x04, 0x04, 0x7C,0x00,0x00,0x00}, //   
	{0x06,0x7C, 0x08, 0x10, 0x08, 0x7C,0x00,0x00,0x00}, //   
	{0x06,0x7C, 0x10, 0x10, 0x10, 0x7C,0x00,0x00,0x00}, //   
	{0x06,0x38, 0x44, 0x44, 0x44, 0x38,0x00,0x00,0x00}, //   
	{0x06,0x7C, 0x04, 0x04, 0x04, 0x7C,0x00,0x00,0x00}, //   
	{0x06,0x7C, 0x14, 0x14, 0x14, 0x08,0x00,0x00,0x00}, //   
	{0x05,0x38, 0x44, 0x44, 0x44, 0x00,0x00,0x00,0x00}, //   
	{0x06,0x04, 0x04, 0x7C, 0x04, 0x04,0x00,0x00,0x00}, //  
	{0x06,0x0C, 0x50, 0x50, 0x50, 0x3C,0x00,0x00,0x00}, //   
	{0x06,0x08, 0x14, 0x7C, 0x14, 0x08,0x00,0x00,0x00}, //   
	{0x06,0x44, 0x28, 0x10, 0x28, 0x44,0x00,0x00,0x00}, //   
	{0x06,0x3C, 0x20, 0x20, 0x3C, 0x60,0x00,0x00,0x00}, //   
	{0x06,0x0C, 0x10, 0x10, 0x10, 0x7C,0x00,0x00,0x00}, //  
	{0x06,0x7C, 0x40, 0x7C, 0x40, 0x7C,0x00,0x00,0x00}, //   
	{0x06,0x3C, 0x20, 0x3C, 0x20, 0x7C,0x00,0x00,0x00}, // 
    {0x06,0x04, 0x7C, 0x50, 0x50, 0x20,0x00,0x00,0x00}, //   
	{0x06,0x7C, 0x50, 0x20, 0x00, 0x7C,0x00,0x00,0x00}, //   
	{0x06,0x00, 0x7C, 0x50, 0x50, 0x20,0x00,0x00,0x00}, //   
	{0x06,0x28, 0x44, 0x54, 0x54, 0x38,0x00,0x00,0x00}, //  
	{0x06,0x7C, 0x10, 0x38, 0x44, 0x38,0x00,0x00,0x00}, //   
	{0x06,0x48, 0x54, 0x34, 0x14, 0x7C,0x00,0x00,0x00}, //       255
    
};

//------------------------------------------------------
 
//------------------------------------------------------
__flash const unsigned char deg12x16[][24] = {
// 0x30 - '0'.
  {0x00, 0xF8, 0xFE, 0x06, 0x03, 0x03, 0x03, 0x03, 0x03, 0x06, 0xFE, 0xF8,
  0x00, 0x07, 0x1F, 0x18, 0x30, 0x30, 0x30, 0x30, 0x30, 0x18, 0x1F, 0x07},
// 0x31 - '1'.
  {0x00, 0x00, 0x00, 0x0c, 0x0c, 0x0e, 0xff, 0xff, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x30, 0x30, 0x30, 0x3f, 0x3f, 0x30, 0x30, 0x30, 0x00},
// 0x32 - '2'.
  {0x00, 0x1c, 0x1e, 0x07, 0x03, 0x03, 0x83, 0xc3, 0xe3, 0x77, 0x3e, 0x1c,
  0x00, 0x30, 0x38, 0x3c, 0x3e, 0x37, 0x33, 0x31, 0x30, 0x30, 0x30, 0x30},
// 0x33 - '3'.
  {0x00, 0x0c, 0x0e, 0x07, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xe7, 0x7e, 0x3c,
  0x00, 0x0c, 0x1c, 0x38, 0x30, 0x30, 0x30, 0x30, 0x30, 0x39, 0x1f, 0x0e},
// 0x34 - '4'.
  {0x00, 0xc0, 0xe0, 0x70, 0x38, 0x1c, 0x0e, 0x07, 0xff, 0xff, 0x00, 0x00,
  0x00, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x3f, 0x3f, 0x03, 0x03},
// 0x35 - '5'.
  {0x00, 0x3f, 0x7f, 0x63, 0x63, 0x63, 0x63, 0x63, 0x63, 0xe3, 0xc3, 0x83,
  0x00, 0x0c, 0x1c, 0x38, 0x30, 0x30, 0x30, 0x30, 0x30, 0x38, 0x1f, 0x0f},
// 0x36 - '6'.
  {0x00, 0xc0, 0xf0, 0xf8, 0xdc, 0xce, 0xc7, 0xc3, 0xc3, 0xc3, 0x80, 0x00,
  0x00, 0x0f, 0x1f, 0x39, 0x30, 0x30, 0x30, 0x30, 0x30, 0x39, 0x1f, 0x0f},
// 0x37 - '7'.
  {0x00, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0xc3, 0xf3, 0x3f, 0x0f, 0x03,
  0x00, 0x00, 0x00, 0x00, 0x30, 0x3c, 0x0f, 0x03, 0x00, 0x00, 0x00, 0x00},
// 0x38 - '8'.
  {0x00, 0x00, 0xbc, 0xfe, 0xe7, 0xc3, 0xc3, 0xc3, 0xe7, 0xfe, 0xbc, 0x00,
  0x00, 0x0f, 0x1f, 0x39, 0x30, 0x30, 0x30, 0x30, 0x30, 0x39, 0x1f, 0x0f},
// 0x39 - '9'.
  {0x00, 0x3c, 0x7e, 0xe7, 0xc3, 0xc3, 0xc3, 0xc3, 0xc3, 0xe7, 0xfe, 0xfc,
  0x00, 0x00, 0x00, 0x30, 0x30, 0x30, 0x38, 0x1c, 0x0e, 0x07, 0x03, 0x00},
// 0x3a - .
  {0x00, 0x00, 0x00, 0x00, 0x70, 0x70, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00,
  0x00, 0x00, 0x00, 0x00, 0x1c, 0x1c, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00}
  };
//------------------------------------------------------
void LCD_setxy(char x,unsigned char y)        //   
{
posx=x;
posy=y;
}
//------------------------------------------------------
void LCD_deg12x16(unsigned char c, unsigned char color)    //  
{ // c = address of character in the font array
char i, j;
unsigned char b;

for(i=0; i<12; i++) 
  {
  b=deg12x16[c][i];
  for(j=0; j<8; j++) 
    {
    LCD_pixel(posx, posy+j, (~(b ^ color)) & 1);
    b >>= 1;
    }
  posx++;
  }
posy+=8;
posx-=12;
for(i=12; i<24; i++) 
  {
  b = deg12x16[c][i];
  for(j=0; j<8; j++) 
    {
    LCD_pixel(posx, posy+j, (~(b ^ color)) & 1);
    b >>= 1;
    }
  posx++;
  }
}
//------------------------------------------------------
unsigned char LCD_get_char_len(unsigned char ch)
{
return font_norm[ch-32][0];
}
//------------------------------------------------------
void LCD_char(unsigned char ch, unsigned char color)    //  
{
unsigned char x, y, len, b;
flash unsigned char *p;


if(ch=='\t') { posx=72; return; }
p=font_norm[ch-32];
len=*p++; if(len>8) len=8;
for(x=0; x<len; x++) 
  {
  b=*p++;
  for(y=0; y<8; y++) 
    {
    LCD_pixel(posx, posy+y, (~(b ^ color)) & 1);
    b >>= 1;
    }
  posx++; 
  }
}
//------------------------------------------------------
void LCD_frame(unsigned char x)   
{
LCD_line(x-1, posy-1, posx-1, posy-1, 1);
LCD_line(x-1, posy-1, x-1, posy+7, 1);
}
//------------------------------------------------------
void  LCD_wrs (unsigned char*str,unsigned char color) //          
{
unsigned char x;
 
x=posx;
while(*str!='\0')  LCD_char(*str++, color);
if(color==0) LCD_frame(x);
}
//------------------------------------------------------
void LCD_wrsf (unsigned char flash*str,unsigned char color) //        
{
unsigned char x;

x=posx;
while(*str!='\0') LCD_char(*str++, color);
if(color==0) LCD_frame(x);
}







